% LAMBERT_GOODING
% April 9, 1991.
% This routine calls the Gooding Lambert routine and gives it the 
% necessary inputs.  It calculates the transfer angle and the radius
% magnitudes that Gooding's algorithm requires.
% Lambert_Gooding written by M.E.Loucks and M.C.Jones
% Uses: mag_v, dot_v, cross_v, norm_v, lamrhg.
%
% This is the driver program that calls the lambert solver.

% lambert_gooding

function [dv1, dv2, dv3, dv4, n, theta] = lambert(pos_1,vel_1,pos_2,vel_2,dt,mu,tol,rev)

% input:
% pos1(3) - position vector for first state
% vel1(3) - velocity vector for first state
% pos1(3) - position vector for second state
% vel2(3) - velocity vector for second state
% dt - time interval between first and second state (same units as velocity times)
% mu - gravitional constant for central body
% tol - accuracy of calculations

% Output:
% dv1(3) - Delta V required at first state
% dv2(3) - Delta V requires at second state
% theta - Angle of transfer (rad)

% specific units are not required as long as you are consistant.  Same units for mu as distance, velocity and time.

%pos_1   = zeros(3,1); vel_1    = zeros(3,1);   % input state (pos, vel) vectors
%pos_2   = zeros(3,1); vel_2    = zeros(3,1);   % input state (pos, vel) vectors
dv1     = zeros(3,1); dv2      = zeros(3,1);   % output delta V vectors
ih_v    = zeros(3,1); ih1_v    = zeros(3,1);   % normals to ang momentum,trans orb
r1_r2_v = zeros(3,1); dum_v    = zeros(3,1); 
ir1_v   = zeros(3,1); ir2_v    = zeros(3,1);   % unit vectors along r1, r2	
trans1_v= zeros(3,1); trans2_v = zeros(3,1);   % transverse vectors at r1, r2

% tol, mu, dt, pi			     !tolerance, grav. param., delta t
% theta, r1, r2, test_dir	     !transf. angle,radius1, radius2
% vr11, vr12, vr21, vr22		 !radial cmpnts of lmbrt sltns
% cos_theta, sin_theta
% vt11, vt12, vt21, vt22		 !transverse cmpnts of lmbrt sltns
% mag_v, dot_v,fix_0_2pi		 !functions called
% integer igtl, n			     !# of iterations, # of solutions

r1 = mag_v(pos_1);
r2 = mag_v(pos_2);
n = 0; % number of solutions
theta = 0;
	  
% Calculate theta if it is not given, note that since this routine
% works for multiple revolution transfers, theta can be specified
% as greater than 2pi.  Thus, the ability to specify theta external
% to this routine has been created.  

if theta == 0.0
  cos_theta = dot(pos_1,pos_2)/(r1*r2);

% Force transfer to be prograde by making angular momenta vectors 	 
% the same direction and between 0 and 2 pi.

  dum_v = cross(pos_1,vel_1);
  ih1_v = norm_v(dum_v);

  dum_v = cross(pos_1,pos_2);
  r1_r2_v = norm_v(dum_v);
				      
% The dot product will tell whether ih1_v and r1_r2_v are in the same  
% direction.

  test_dir = dot(ih1_v, r1_r2_v);

% Here we test for retrograde transfers and disallow them.  Note that
% if the angular momentum unit vector ih1_v is not in the same direction
% as r1_r2_v, then the cross product of r1 and r2 has given us a negative
% transfer angle who's absolute value is less than pi.  So we must force
% this to be a positve angle and force the normal to the plane(r1_r2_v)
% to point in the right direction as well.  If the dot product of ih1_v
% and r1_r2_v is negative, then the condition exists and the sin of theta
% and r1_r2_v must be adjusted accordingly.

  if test_dir >= 0.0				    %0 < theta < 180 degrees
    sin_theta = sqrt(1.0 - cos_theta*cos_theta);
  else									%180 < theta < 360 degrees

% Force the angle to be greater than pi and force r1_r2_v in the other 
% direction for prograde transfers only.

    sin_theta = -sqrt(1.0 - cos_theta*cos_theta);
	r1_r2_v(1) = -r1_r2_v(1);
	r1_r2_v(2) = -r1_r2_v(2);
	r1_r2_v(3) = -r1_r2_v(3);
  end                                   % IF block end
      
  theta = atan2(sin_theta, cos_theta);
  theta = fix_0_2pi(theta);
  
end                                     % IF block end

theta = theta + rev*2*pi;
	   
[vr11, vt11, vr12, vt12, vr21, vt21, vr22, vt22, n, igtl] = lamrhg(mu, r1, r2, theta, dt, n);
	 
% Create unit vectors along the radii

ir1_v = norm_v(pos_1);
ir2_v = norm_v(pos_2);
 	  
% Create transverse unit vectors at both ends

if mag_v(r1_r2_v) == 0.0
  trans1_v = cross(ih1_v,ir1_v);
  trans2_v = cross(ih1_v,ir2_v);
else
  trans1_v = cross(r1_r2_v,ir1_v);
  trans2_v = cross(r1_r2_v,ir2_v);
end
	  
% assume only one solution is coming out and so only values in 
% vr11, vr12, vt11, vt12 are used

dv1(1) = vt11*trans1_v(1) + vr11*ir1_v(1) - vel_1(1);
dv1(2) = vt11*trans1_v(2) + vr11*ir1_v(2) - vel_1(2);
dv1(3) = vt11*trans1_v(3) + vr11*ir1_v(3) - vel_1(3);
	  
dv2(1) = vt12*trans2_v(1) + vr12*ir2_v(1) - vel_2(1);
dv2(2) = vt12*trans2_v(2) + vr12*ir2_v(2) - vel_2(2);
dv2(3) = vt12*trans2_v(3) + vr12*ir2_v(3) - vel_2(3);

% second solution
dv3(1) = vt21*trans1_v(1) + vr21*ir1_v(1) - vel_1(1);
dv3(2) = vt21*trans1_v(2) + vr21*ir1_v(2) - vel_1(2);
dv3(3) = vt21*trans1_v(3) + vr21*ir1_v(3) - vel_1(3);
	  
dv4(1) = vt22*trans2_v(1) + vr22*ir2_v(1) - vel_2(1);
dv4(2) = vt22*trans2_v(2) + vr22*ir2_v(2) - vel_2(2);
dv4(3) = vt22*trans2_v(3) + vr22*ir2_v(3) - vel_2(3);

return
