% SUBROUTINE LAMRGH (VLAMB)
% Dec 24, 1990
% Subroutine Lamrhg obtained from Alan Klumpp of JPL
% Subroutine Lamrhg is by Robert H. Gooding of the Royal Aerospace
% Establshment, Farnborough, Hants, England
% This routine solves Lambert's problem using a method developed by RH Gooding.
%
%    1.   90 10 26, Allan Klumpp:  Renamed VLAMB LAMRHG, XLAMB LAMGXL,
%         and TLAMB LAMGTL.
%
%    2.   90 11 22, Allan Klumpp:  Changed the computation of the number
%         of complete revs M and half the remaining central angle THR2
%         so that the algorithm would work for a hyperbolic transfer
%         thru a  central angle of exactly 2 PI.  Now in this case,
%         M = 0 and THR2 = PI instead of M = 1 and THR2 = 0.0.
%
%    3.   90 12 12, Allan Klumpp:  Added IGTL to calling sequences.
%         IGTL counts the number of times LAMGTL is called.
%    

function [vr11, vt11, vr12, vt12, vr21, vt21, vr22, vt22, n, igtl] = lamrhg(gm, r1, r2, th, tdelt, n);

% Input:
% gm - gravitational constant
% r1 - radius at first terminal
% r2 - radius at second terminal
% th - transfer angle
% n = number of solutions

% Output:
% vr11 - radial velocity at first terminal
% vt11 - transverse velocity at first terminal
% vr12 - radial velocity at second terminal
% vt12 - Transverse velocity at seconds terminal
% vr21, vt21, vr22, vt22 Radial and transverse velocties for second solultion (multiple rev transfers only)
% igtl - counter for Lamgtl()

twopi = 2.0*pi;    
unused = 0.0;
x2 = 0.0; %avoid non-assignment message when running

igtl = 0;          % counter for calls to lamgtl
                                                      
%      the following yields m = 0 when th = 2 pi exactly   
%      neither this nor the original code works for        
%      th < 0.0                                            

thr2 = th;
m = 0;

vr11 =0.0;
vt11 =0.0;
vr12 =0.0;
vt12 =0.0;
vr21 =0.0;
vt21 =0.0;
vr22 =0.0;
vt22 =0.0;

while thr2 > twopi                  %Count number of Revs in th
   thr2 = thr2 - twopi;
   m = m + 1;
end

thr2 = thr2 / 2.0;

dr = r1 - r2;
r1r2 = r1*r2;
r1r2th = 4.0*r1r2*sin(thr2)*sin(thr2);
csq = dr*dr + r1r2th;
c = sqrt(csq);
s = (r1 + r2 + c)/2.0;
gms = sqrt(gm*s/2.0);
qsqfm1 = c/s;
q = sqrt(r1r2)*cos(thr2)/s;

if  c ~= 0.0
  rho = dr/c;
  sig = r1r2th/csq;
else
  rho = 0.0;
  sig = 1.0;
end

t = 4.0*gms*tdelt/(s*s);

[x1, x2, n, igtl] = lamgxl (m, q, qsqfm1, t, n, igtl);

%  proceed for single solution, or a pair
for i = 1:n

    if i == 1.0 
        x = x1;
    else
        x = x2;
    end

[unused, qzminx, qzplx, zplqx,igtl] = lamgtl(m, q, qsqfm1, x, -1, igtl);

vt2 = gms*zplqx*sqrt(sig);
vr1 = gms*(qzminx - qzplx*rho)/r1;
vt1 = vt2/r1;
vr2 = -gms*(qzminx + qzplx*rho)/r2;
vt2 = vt2/r2;

    if i == 1.0
        vr11 = vr1;
        vt11 = vt1;
        vr12 = vr2;
        vt12 = vt2;
    else
        vr21 = vr1;
        vt21 = vt1;
        vr22 = vr2;
        vt22 = vt2;
    end
end                     % For Loop End

return

